@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: FLEXI-STREAMS; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/flexi-streams/in-memory.lisp,v 1.13 2006/09/15 07:21:41 edi Exp $
+;;; $Header: /usr/local/cvsrep/flexi-streams/in-memory.lisp,v 1.15 2006/11/05 23:45:37 edi Exp $
 
 ;;; Copyright (c) 2005-2006, Dr. Edmund Weitz.  All rights reserved.
 
@@ -43,11 +43,21 @@ octets from or writes octets to a sequence in RAM."))
   (:documentation "An IN-MEMORY-INPUT-STREAM is a binary stream that
 reads octets from a sequence in RAM."))
 
+#+:cmu
+(defmethod output-stream-p ((stream in-memory-input-stream))
+  "Explicitely states whether this is an output stream."
+  nil)
+
 (defclass in-memory-output-stream (in-memory-stream fundamental-binary-output-stream)
   ()
   (:documentation "An IN-MEMORY-OUTPUT-STREAM is a binary stream that
 writes octets to a sequence in RAM."))
 
+#+:cmu
+(defmethod input-stream-p ((stream in-memory-output-stream))
+  "Explicitely states whether this is an input stream."
+  nil)
+
 (defclass list-stream ()
   ((list :initarg :list
          :accessor list-stream-list
@@ -127,16 +137,10 @@ to read from or write to a closed IN-MEMORY stream."))
   (or (pop (list-stream-list stream))
       :eof))
 
-(defmethod stream-check-eof-no-hang ((stream list-input-stream))
-  "EOF means that the list is empty."
-  (check-if-open stream)
-  (cond ((list-stream-list stream) nil)
-        (t :eof)))
-
 (defmethod stream-listen ((stream list-input-stream))
-  "Same as checking for EOF."
+  "Checks whether list is not empty."
   (check-if-open stream)
-  (not (eq (stream-check-eof-no-hang stream) :eof)))
+  (list-stream-list stream))
 
 (defmethod stream-read-sequence ((stream list-input-stream) sequence start end &key)
   "Repeatedly pops elements from the list until it's empty."
@@ -156,17 +160,10 @@ END pointer."
            (aref (vector-stream-vector stream) index))
           (t :eof))))
 
-(defmethod stream-check-eof-no-hang ((stream vector-input-stream))
-  "EOF means that INDEX is beyond END."
-  (check-if-open stream)
-  (cond ((< (vector-stream-index stream)
-            (vector-stream-end stream)) nil)
-        (t :eof)))
-  
 (defmethod stream-listen ((stream vector-input-stream))
-  "Same as checking for EOF."
+  "Checking whether INDEX is beyond END."
   (check-if-open stream)
-  (not (eq (stream-check-eof-no-hang stream) :eof)))
+  (< (vector-stream-index stream) (vector-stream-end stream)))
   
 (defmethod stream-read-sequence ((stream vector-input-stream) sequence start end &key)
   "Traverses both sequences in parallel until the end of one of them
