@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: FLEXI-STREAMS; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/flexi-streams/input.lisp,v 1.17 2006/09/04 08:32:06 edi Exp $
+;;; $Header: /usr/local/cvsrep/flexi-streams/input.lisp,v 1.22 2006/11/14 20:53:16 edi Exp $
 
 ;;; Copyright (c) 2005-2006, Dr. Edmund Weitz.  All rights reserved.
 
@@ -29,9 +29,22 @@
 
 (in-package :flexi-streams)
 
+#-:lispworks
 (defun read-byte* (flexi-input-stream &optional eof-value)
   "Reads one byte \(octet) from the underlying stream of
 FLEXI-OUTPUT-STREAM \(or from the internal stack if it's not
+empty)."
+  (incf (flexi-stream-position flexi-input-stream))
+  (or (pop (flexi-stream-octet-stack flexi-input-stream))
+      (let ((octet (read-byte (flexi-stream-stream flexi-input-stream) nil nil)))
+        (cond (octet)
+              (t (decf (flexi-stream-position flexi-input-stream))
+                 eof-value)))))
+
+#+:lispworks
+(defmethod read-byte* (flexi-input-stream &optional eof-value)
+  "Reads one byte \(octet) from the underlying stream of
+FLEXI-OUTPUT-STREAM \(or from the internal stack if it's not
 empty)."
   (incf (flexi-stream-position flexi-input-stream))
   (or (pop (flexi-stream-octet-stack flexi-input-stream))
@@ -44,6 +57,19 @@ empty)."
                eof-value)
               (t (aref buffer 0))))))
 
+#+:lispworks
+(defmethod read-byte* ((flexi-input-stream flexi-binary-input-stream) &optional eof-value)
+  "Reads one byte \(octet) from the underlying stream of
+FLEXI-OUTPUT-STREAM \(or from the internal stack if it's not empty).
+Optimized version \(only needed for LispWorks) in case the underlying
+stream is binary."
+  (incf (flexi-stream-position flexi-input-stream))
+  (or (pop (flexi-stream-octet-stack flexi-input-stream))
+      (let ((octet (read-byte (flexi-stream-stream flexi-input-stream) nil nil)))
+        (cond (octet)
+              (t (decf (flexi-stream-position flexi-input-stream))
+                 eof-value)))))
+
 (defun read-char-8-bit (flexi-input-stream encoding-table)
   "Reads one octet from the stream underlying FLEXI-INPUT-STREAM
 and converts it into a character code using the array
@@ -55,8 +81,8 @@ ENCODING-TABLE."
                    (not *use-replacement-char*))
               (null char-code))
       (signal-encoding-error flexi-input-stream
-                              "No character which correspondes to octet #x~X."
-                              octet))
+                             "No character which correspondes to octet #x~X."
+                             octet))
     char-code))
 
 (defun read-char-utf-32 (flexi-input-stream)
@@ -70,7 +96,7 @@ current external format of FLEXI-INPUT-STREAM."
                  (or (read-byte* flexi-input-stream)
                      (cond (first-octet-seen
                             (signal-encoding-error flexi-input-stream
-                                                    "End of file while in UTF-32 sequence."))
+                                                   "End of file while in UTF-32 sequence."))
                            (t (return-from read-char-utf-32 :eof))))
                (setq first-octet-seen t))))
       (loop for count in (if (external-format-little-endian
@@ -91,7 +117,7 @@ current external format of FLEXI-INPUT-STREAM."
                    (or (read-byte* flexi-input-stream)
                        (cond (first-octet-seen
                               (signal-encoding-error flexi-input-stream
-                                                      "End of file while in UTF-16 sequence."))
+                                                     "End of file while in UTF-16 sequence."))
                              (t (return-from read-char-utf-16 :eof))))
                  (setq first-octet-seen t)))
              (read-next-word ()
@@ -142,7 +168,7 @@ determined by the UTF-8 encoding."
                    (values (logand octet #b00000001) 5))
                   (t (signal-encoding-error flexi-input-stream
                                             "Unexpected value #x~X at start of UTF-8 sequence."
-                                             octet)))
+                                            octet)))
           ;; note that we currently don't check for "overlong"
           ;; sequences or other illegal values
           (loop for result = start then (+ (ash result 6)
@@ -152,7 +178,7 @@ determined by the UTF-8 encoding."
                 unless (= #b10000000 (logand octet #b11000000))
                 do (signal-encoding-error flexi-input-stream
                                           "Unexpected value #x~X in UTF-8 sequence."
-                                           octet)
+                                          octet)
                 finally (return result)))))))
 
 (defmethod stream-clear-input ((stream flexi-input-stream))
@@ -160,14 +186,14 @@ determined by the UTF-8 encoding."
 and also clears the value of the OCTET-STACK slot."
   ;; note that we don't reset the POSITION slot
   (setf (flexi-stream-octet-stack stream) nil)
-  (stream-clear-input (flexi-stream-stream stream)))
+  (clear-input (flexi-stream-stream stream)))
 
 (defmethod stream-listen ((stream flexi-input-stream))
   "Calls the corresponding method for the underlying input stream
 but first check if \(old) input is available in the OCTET-STACK
 slot."
   (or (flexi-stream-octet-stack stream)
-      (stream-listen (flexi-stream-stream stream))))
+      (listen (flexi-stream-stream stream))))
 
 (defmethod stream-read-byte ((stream flexi-input-stream))
   "Reads one byte \(octet) from the underlying stream."
@@ -207,20 +233,36 @@ current EOL style."
              "This internal function does all the work of reading
 from the stream and converting to character codes but it doesn't
 handle line endings."
-             (cond ((ascii-name-p external-format-name)
-                    (read-char-8-bit stream +ascii-table+))
-                   ((iso-8859-name-p external-format-name)
-                    (read-char-8-bit stream
-                                     (cdr (assoc external-format-name +iso-8859-tables+
-                                                 :test #'eq))))
-                   ((code-page-name-p external-format-name)
-                    (read-char-8-bit stream
-                                     (cdr (assoc (external-format-id external-format)
-                                                 +code-page-tables+))))
-                   (t (case external-format-name
-                        (:utf-8 (read-char-utf-8 stream))
-                        (:utf-16 (read-char-utf-16 stream))
-                        (:utf-32 (read-char-utf-32 stream)))))))
+             (restart-case
+                 (handler-bind ((flexi-stream-encoding-error (lambda (condition)
+                                                               (declare (ignore condition))
+                                                               (when *substitution-char*
+                                                                 (use-value *substitution-char*)))))
+                   (cond ((ascii-name-p external-format-name)
+                          (read-char-8-bit stream +ascii-table+))
+                         ((koi8-r-name-p external-format-name)
+                          (read-char-8-bit stream +koi8-r-table+))
+                         ((iso-8859-name-p external-format-name)
+                          (read-char-8-bit stream
+                                           (cdr (assoc external-format-name +iso-8859-tables+
+                                                       :test #'eq))))
+                         ((code-page-name-p external-format-name)
+                          (read-char-8-bit stream
+                                           (cdr (assoc (external-format-id external-format)
+                                                       +code-page-tables+))))
+                         (t (case external-format-name
+                              (:utf-8 (read-char-utf-8 stream))
+                              (:utf-16 (read-char-utf-16 stream))
+                              (:utf-32 (read-char-utf-32 stream))))))
+               (use-value (char)
+                 :report "Specify a character to be used instead."
+                 :interactive (lambda ()
+                                (loop
+                                 (format *query-io* "Type a character: ")
+                                 (let ((line (read-line *query-io*)))
+                                   (when (= 1 (length line))
+                                     (return (list (char line 0)))))))
+                 (char-code char)))))
       (let ((char-code (get-char-code)))
         (when (eq char-code :eof)
           (return-from stream-read-char :eof))
@@ -257,18 +299,32 @@ octet available."
        (stream-read-char stream)))
 
 (defmethod stream-read-sequence ((stream flexi-input-stream) sequence start end &key)
-  "Reads enough input from STREAM to fill SEQUENCE from START to
-END.  If SEQUENCE is an array which can store octets we use
-STREAM-READ-BYTE to fill it, otherwise we use STREAM-READ-CHAR."
-  (let ((reader (cond ((and (arrayp sequence)
-                            (subtypep 'octet (array-element-type sequence)))
-                       #'stream-read-byte)
-                      (t #'stream-read-char))))
-    (loop for index from start below end
-          for element = (funcall reader stream)
-          until (eq element :eof)
-          do (setf (elt sequence index) element)
-          finally (return index))))
+  "Reads enough input from STREAM to fill SEQUENCE from START to END.
+If SEQUENCE is an array which can store octets we use READ-SEQUENCE to
+fill it in one fell swoop, otherwise we iterate using
+STREAM-READ-CHAR."
+  (cond ((and (arrayp sequence)
+              (subtypep 'octet (array-element-type sequence)))
+         (setf (flexi-stream-last-char stream) nil)
+         (let ((cursor start))
+           (loop with octet-stack = (flexi-stream-octet-stack stream)
+                 for continuep = (< cursor end)
+                 for octet = (and continuep (pop octet-stack))
+                 while octet
+                 do (setf (aref sequence cursor) octet)
+                    (incf cursor))
+           (let ((index
+                  (read-sequence sequence (flexi-stream-stream stream) :start cursor :end end)))
+             (incf (flexi-stream-position stream) (- index start))
+             (when (> index start)
+               (setf (flexi-stream-last-octet stream) (aref sequence (1- index))))
+             index)))
+        (t
+         (loop for index from start below end
+               for element = (stream-read-char stream)
+               until (eq element :eof)
+               do (setf (elt sequence index) element)
+               finally (return index)))))
 
 (defmethod stream-unread-char ((stream flexi-input-stream) char)
   "Implements UNREAD-CHAR for streams of type FLEXI-INPUT-STREAM.
