@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: FLEXI-STREAMS; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/flexi-streams/specials.lisp,v 1.11 2006/08/10 14:47:03 edi Exp $
+;;; $Header: /usr/local/cvsrep/flexi-streams/specials.lisp,v 1.13 2006/11/14 20:53:16 edi Exp $
 
 ;;; Copyright (c) 2005-2006, Dr. Edmund Weitz.  All rights reserved.
 
@@ -43,6 +43,7 @@
     (:ucs4 . :utf-32)
     (:ucs-4 . :utf-32)
     (:ascii . :us-ascii)
+    (:koi8r . :koi8-r)
     (:latin-1 . :iso-8859-1)
     (:latin1 . :iso-8859-1)
     (:latin-2 . :iso-8859-2)
@@ -127,6 +128,11 @@ the code is compiled on.")
 return the replacement character (65533) instead of signalling an
 error.")
 
+(defvar *substitution-char* nil
+  "If this value is not NIL, it should be a character which is used
+\(as if by a USE-VALUE restart) whenever during reading an error of
+type FLEXI-STREAM-ENCODING-ERROR would have been signaled otherwise.")
+
 ;; stuff for Nikodemus Siivola's HYPERDOC
 ;; see <http://common-lisp.net/project/hyperdoc/>
 ;; and <http://www.cliki.net/hyperdoc>
