@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: FLEXI-STREAMS; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/flexi-streams/util.lisp,v 1.8 2006/06/29 13:34:00 edi Exp $
+;;; $Header: /usr/local/cvsrep/flexi-streams/util.lisp,v 1.9 2006/10/31 14:09:08 edi Exp $
 
 ;;; Copyright (c) 2005-2006, Dr. Edmund Weitz.  All rights reserved.
 
@@ -115,6 +115,10 @@ signals an error otherwise."
   "Checks whether NAME is the keyword :ASCII."
   (eq name :us-ascii))
 
+(defun koi8-r-name-p (name)
+  "Checks whether NAME is the keyword :KOI8-R."
+  (eq name :koi8-r))
+
 (defun code-page-name-p (name)
   "Checks whether NAME is the keyword :CODE-PAGE."
   (eq name :code-page))
